#ifndef __BOOST__IO__FORMATLIST_T__HPP
#define __BOOST__IO__FORMATLIST_T__HPP
#  include <iostream>

   namespace boost { namespace io
   {
      template< typename ForwardIterator, typename FormatType = char >
      class formatlist_t
      {
         public:
            typedef FormatType                             listfmt_type;
            typedef format_traits< listfmt_type >          traits_type;
         private: // list format control
            listfmt_type               __openlist;
            listfmt_type               __closelist;
            listfmt_type               __separator;
         public:
            inline listfmt_type                  openlist()  const throw();
            inline listfmt_type                  closelist() const throw();
            inline listfmt_type                  separator() const throw();
         public:
            inline formatlist_t &                format
                                                 (
                                                    listfmt_type, // open  list
                                                    listfmt_type, // close list
                                                    listfmt_type  // seperator
                                                 ) throw();
            inline formatlist_t &                format
                                                 (
                                                    listfmt_type, // open  list
                                                    listfmt_type  // close list
                                                 ) throw();
            inline formatlist_t &                format
                                                 (
                                                    listfmt_type  // seperator
                                                 ) throw();
         private: // spacing control
            bool                       __spaceBeforeSep;
            bool                       __spaceAfterSep;
            bool                       __spaceAroundList;
         public:
            inline bool                          spaceBeforeSep()  const throw();
            inline bool                          spaceAfterSep()   const throw();
            inline bool                          spaceAroundList() const throw();
         public:
            inline formatlist_t &                space( bool, bool, bool ) throw(); // before, after, around
            inline formatlist_t &                space( bool, bool )       throw(); // before, after
            inline formatlist_t &                space( bool )             throw(); // around
         private: // list range
            ForwardIterator            first;
            ForwardIterator            last;
         public:
            inline ForwardIterator               begin() const throw();
            inline ForwardIterator               end()   const throw();
         public:
            typedef ForwardIterator                        iterator;
         public:
            inline           formatlist_t( ForwardIterator, ForwardIterator );
      };

      template
      <
         typename CharT, class TraitsT,
         typename ForwardIterator, typename FormatType
      >
      inline std::basic_ostream< CharT, TraitsT > & operator<<
      (
         std::basic_ostream< CharT, TraitsT >  & os,
         formatlist_t< ForwardIterator, FormatType > const & fi
      );
   }}

#  include "formatlist_t.ipp"
#endif
